@@ -67,9 +67,16 @@ class XdsApi {
     virtual absl::Status ProcessAdsResponseFields(AdsResponseFields fields) = 0;
 
     // Called to parse each individual resource in the ADS response.
+    // Note that resource_name is non-empty only when the resource was
+    // wrapped in a Resource wrapper proto.
     virtual void ParseResource(upb_Arena* arena, size_t idx,
                                absl::string_view type_url,
+                               absl::string_view resource_name,
                                absl::string_view serialized_resource) = 0;
+
+    // Called when a resource is wrapped in a Resource wrapper proto but
+    // we fail to deserialize the wrapper proto.
+    virtual void ResourceWrapperParsingFailed(size_t idx) = 0;
   };
 
   struct ClusterLoadReport {
@@ -140,12 +147,11 @@ class XdsApi {
                 "");
 
   XdsApi(XdsClient* client, TraceFlag* tracer, const XdsBootstrap::Node* node,
-         upb::SymbolTable* symtab);
+         upb::SymbolTable* symtab, std::string user_agent_name,
+         std::string user_agent_version);
 
   // Creates an ADS request.
-  // Takes ownership of \a error.
-  std::string CreateAdsRequest(const XdsBootstrap::XdsServer& server,
-                               absl::string_view type_url,
+  std::string CreateAdsRequest(absl::string_view type_url,
                                absl::string_view version,
                                absl::string_view nonce,
                                const std::vector<std::string>& resource_names,
@@ -153,12 +159,11 @@ class XdsApi {
 
   // Returns non-OK when failing to deserialize response message.
   // Otherwise, all events are reported to the parser.
-  absl::Status ParseAdsResponse(const XdsBootstrap::XdsServer& server,
-                                absl::string_view encoded_response,
+  absl::Status ParseAdsResponse(absl::string_view encoded_response,
                                 AdsResponseParserInterface* parser);
 
   // Creates an initial LRS request.
-  std::string CreateLrsInitialRequest(const XdsBootstrap::XdsServer& server);
+  std::string CreateLrsInitialRequest();
 
   // Creates an LRS request sending a client-side load report.
   std::string CreateLrsRequest(ClusterLoadReportMap cluster_load_report_map);
@@ -179,7 +184,6 @@ class XdsApi {
   TraceFlag* tracer_;
   const XdsBootstrap::Node* node_;  // Do not own.
   upb::SymbolTable* symtab_;        // Do not own.
-  const std::string build_version_;
   const std::string user_agent_name_;
   const std::string user_agent_version_;
 };
