@@ -17,18 +17,21 @@
 
 #include <stdint.h>
 
-#include <algorithm>
+#include <atomic>
 #include <memory>
 
-#include "absl/memory/memory.h"
+#include "absl/status/status.h"
 #include "absl/status/statusor.h"
-#include "absl/strings/str_cat.h"
+#include "absl/strings/str_format.h"
 
-#include <grpc/impl/codegen/gpr_types.h>
+#include <grpc/event_engine/event_engine.h>
+#include <grpc/status.h>
 #include <grpc/support/log.h>
 #include <grpc/support/sync.h>
-#include <grpc/support/time.h>
 
+#include "src/core/lib/event_engine/poller.h"
+#include "src/core/lib/event_engine/time_util.h"
+#include "src/core/lib/gprpp/crash.h"
 #include "src/core/lib/iomgr/port.h"
 
 // This polling engine is only relevant on linux kernels supporting epoll
@@ -36,95 +39,118 @@
 #ifdef GRPC_LINUX_EPOLL
 #include <errno.h>
 #include <limits.h>
-#include <string.h>
 #include <sys/epoll.h>
 #include <sys/socket.h>
 #include <unistd.h>
 
-#include <vector>
-
-#include "absl/synchronization/mutex.h"
-
 #include "src/core/lib/event_engine/posix_engine/event_poller.h"
 #include "src/core/lib/event_engine/posix_engine/lockfree_event.h"
 #include "src/core/lib/event_engine/posix_engine/posix_engine_closure.h"
 #include "src/core/lib/event_engine/posix_engine/wakeup_fd_posix.h"
 #include "src/core/lib/event_engine/posix_engine/wakeup_fd_posix_default.h"
 #include "src/core/lib/gprpp/fork.h"
-#include "src/core/lib/gprpp/time.h"
-
-using ::grpc_event_engine::posix_engine::LockfreeEvent;
-using ::grpc_event_engine::posix_engine::WakeupFd;
+#include "src/core/lib/gprpp/status_helper.h"
+#include "src/core/lib/gprpp/strerror.h"
+#include "src/core/lib/gprpp/sync.h"
 
 #define MAX_EPOLL_EVENTS_HANDLED_PER_ITERATION 1
 
 namespace grpc_event_engine {
-namespace posix_engine {
+namespace experimental {
 
 class Epoll1EventHandle : public EventHandle {
  public:
   Epoll1EventHandle(int fd, Epoll1Poller* poller)
       : fd_(fd),
-        pending_actions_(0),
-        list_(),
+        list_(this),
         poller_(poller),
-        read_closure_(absl::make_unique<LockfreeEvent>(poller->GetScheduler())),
-        write_closure_(
-            absl::make_unique<LockfreeEvent>(poller->GetScheduler())),
+        read_closure_(std::make_unique<LockfreeEvent>(poller->GetScheduler())),
+        write_closure_(std::make_unique<LockfreeEvent>(poller->GetScheduler())),
         error_closure_(
-            absl::make_unique<LockfreeEvent>(poller->GetScheduler())) {
+            std::make_unique<LockfreeEvent>(poller->GetScheduler())) {
     read_closure_->InitEvent();
     write_closure_->InitEvent();
     error_closure_->InitEvent();
-    pending_actions_ = 0;
+    pending_read_.store(false, std::memory_order_relaxed);
+    pending_write_.store(false, std::memory_order_relaxed);
+    pending_error_.store(false, std::memory_order_relaxed);
   }
-  Epoll1Poller* Poller() { return poller_; }
-  void SetPendingActions(bool pending_read, bool pending_write,
+  void ReInit(int fd) {
+    fd_ = fd;
+    read_closure_->InitEvent();
+    write_closure_->InitEvent();
+    error_closure_->InitEvent();
+    pending_read_.store(false, std::memory_order_relaxed);
+    pending_write_.store(false, std::memory_order_relaxed);
+    pending_error_.store(false, std::memory_order_relaxed);
+  }
+  Epoll1Poller* Poller() override { return poller_; }
+  bool SetPendingActions(bool pending_read, bool pending_write,
                          bool pending_error) {
-    pending_actions_ |= pending_read;
+    // Another thread may be executing ExecutePendingActions() at this point
+    // This is possible for instance, if one instantiation of Work(..) sets
+    // an fd to be readable while the next instantiation of Work(...) may
+    // set the fd to be writable. While the second instantiation is running,
+    // ExecutePendingActions() of the first instantiation may execute in
+    // parallel and read the pending_<***>_ variables. So we need to use
+    // atomics to manipulate pending_<***>_ variables.
+
+    if (pending_read) {
+      pending_read_.store(true, std::memory_order_release);
+    }
+
     if (pending_write) {
-      pending_actions_ |= (1 << 2);
+      pending_write_.store(true, std::memory_order_release);
     }
+
     if (pending_error) {
-      pending_actions_ |= (1 << 3);
+      pending_error_.store(true, std::memory_order_release);
     }
+
+    return pending_read || pending_write || pending_error;
   }
   int WrappedFd() override { return fd_; }
-  void OrphanHandle(IomgrEngineClosure* on_done, int* release_fd,
+  void OrphanHandle(PosixEngineClosure* on_done, int* release_fd,
                     absl::string_view reason) override;
   void ShutdownHandle(absl::Status why) override;
-  void NotifyOnRead(IomgrEngineClosure* on_read) override;
-  void NotifyOnWrite(IomgrEngineClosure* on_write) override;
-  void NotifyOnError(IomgrEngineClosure* on_error) override;
+  void NotifyOnRead(PosixEngineClosure* on_read) override;
+  void NotifyOnWrite(PosixEngineClosure* on_write) override;
+  void NotifyOnError(PosixEngineClosure* on_error) override;
   void SetReadable() override;
   void SetWritable() override;
   void SetHasError() override;
   bool IsHandleShutdown() override;
-  void ExecutePendingActions() override {
-    if (pending_actions_ & 1UL) {
+  inline void ExecutePendingActions() {
+    // These may execute in Parallel with ShutdownHandle. Thats not an issue
+    // because the lockfree event implementation should be able to handle it.
+    if (pending_read_.exchange(false, std::memory_order_acq_rel)) {
       read_closure_->SetReady();
     }
-    if ((pending_actions_ >> 2) & 1UL) {
+    if (pending_write_.exchange(false, std::memory_order_acq_rel)) {
       write_closure_->SetReady();
     }
-    if ((pending_actions_ >> 3) & 1UL) {
+    if (pending_error_.exchange(false, std::memory_order_acq_rel)) {
       error_closure_->SetReady();
     }
-    pending_actions_ = 0;
   }
-  absl::Mutex* mu() { return &mu_; }
+  grpc_core::Mutex* mu() { return &mu_; }
   LockfreeEvent* ReadClosure() { return read_closure_.get(); }
   LockfreeEvent* WriteClosure() { return write_closure_.get(); }
   LockfreeEvent* ErrorClosure() { return error_closure_.get(); }
   Epoll1Poller::HandlesList& ForkFdListPos() { return list_; }
+  ~Epoll1EventHandle() override = default;
 
  private:
   void HandleShutdownInternal(absl::Status why, bool releasing_fd);
   // See Epoll1Poller::ShutdownHandle for explanation on why a mutex is
   // required.
-  absl::Mutex mu_;
+  grpc_core::Mutex mu_;
   int fd_;
-  int pending_actions_;
+  // See Epoll1Poller::SetPendingActions for explanation on why pending_<***>_
+  // need to be atomic.
+  std::atomic<bool> pending_read_{false};
+  std::atomic<bool> pending_write_{false};
+  std::atomic<bool> pending_error_{false};
   Epoll1Poller::HandlesList list_;
   Epoll1Poller* poller_;
   std::unique_ptr<LockfreeEvent> read_closure_;
@@ -206,20 +232,6 @@ void ForkPollerListRemovePoller(Epoll1Poller* poller) {
   }
 }
 
-int PollDeadlineToMillisTimeout(grpc_core::Timestamp millis) {
-  if (millis == grpc_core::Timestamp::InfFuture()) return -1;
-  grpc_core::Timestamp now =
-      grpc_core::Timestamp::FromTimespecRoundDown(gpr_now(GPR_CLOCK_MONOTONIC));
-  int64_t delta = (millis - now).millis();
-  if (delta > INT_MAX) {
-    return INT_MAX;
-  } else if (delta < 0) {
-    return 0;
-  } else {
-    return static_cast<int>(delta);
-  }
-}
-
 bool InitEpoll1PollerLinux();
 
 // Called by the child process's post-fork handler to close open fds,
@@ -252,7 +264,7 @@ void ResetEventManagerOnFork() {
 // It is possible that GLIBC has epoll but the underlying kernel doesn't.
 // Create epoll_fd to make sure epoll support is available
 bool InitEpoll1PollerLinux() {
-  if (!grpc_event_engine::posix_engine::SupportsWakeupFd()) {
+  if (!grpc_event_engine::experimental::SupportsWakeupFd()) {
     return false;
   }
   int fd = EpollCreateAndCloexec();
@@ -269,7 +281,7 @@ bool InitEpoll1PollerLinux() {
 
 }  // namespace
 
-void Epoll1EventHandle::OrphanHandle(IomgrEngineClosure* on_done,
+void Epoll1EventHandle::OrphanHandle(PosixEngineClosure* on_done,
                                      int* release_fd,
                                      absl::string_view reason) {
   bool is_release_fd = (release_fd != nullptr);
@@ -290,17 +302,18 @@ void Epoll1EventHandle::OrphanHandle(IomgrEngineClosure* on_done,
   {
     // See Epoll1Poller::ShutdownHandle for explanation on why a mutex is
     // required here.
-    absl::MutexLock lock(&mu_);
+    grpc_core::MutexLock lock(&mu_);
     read_closure_->DestroyEvent();
     write_closure_->DestroyEvent();
     error_closure_->DestroyEvent();
   }
-
+  pending_read_.store(false, std::memory_order_release);
+  pending_write_.store(false, std::memory_order_release);
+  pending_error_.store(false, std::memory_order_release);
   {
-    absl::MutexLock lock(&poller_->mu_);
+    grpc_core::MutexLock lock(&poller_->mu_);
     poller_->free_epoll1_handles_list_.push_back(this);
   }
-
   if (on_done != nullptr) {
     on_done->SetStatus(absl::OkStatus());
     poller_->GetScheduler()->Run(on_done);
@@ -312,6 +325,8 @@ void Epoll1EventHandle::OrphanHandle(IomgrEngineClosure* on_done,
 // shutdown() syscall on that fd)
 void Epoll1EventHandle::HandleShutdownInternal(absl::Status why,
                                                bool releasing_fd) {
+  grpc_core::StatusSetInt(&why, grpc_core::StatusIntProperty::kRpcStatus,
+                          GRPC_STATUS_UNAVAILABLE);
   if (read_closure_->SetShutdown(why)) {
     if (!releasing_fd) {
       shutdown(fd_, SHUT_RDWR);
@@ -319,7 +334,8 @@ void Epoll1EventHandle::HandleShutdownInternal(absl::Status why,
       epoll_event phony_event;
       if (epoll_ctl(poller_->g_epoll_set_.epfd, EPOLL_CTL_DEL, fd_,
                     &phony_event) != 0) {
-        gpr_log(GPR_ERROR, "epoll_ctl failed: %s", strerror(errno));
+        gpr_log(GPR_ERROR, "epoll_ctl failed: %s",
+                grpc_core::StrError(errno).c_str());
       }
     }
     write_closure_->SetShutdown(why);
@@ -355,7 +371,7 @@ Epoll1Poller::~Epoll1Poller() {
     g_epoll_set_.epfd = -1;
   }
   {
-    absl::MutexLock lock(&mu_);
+    grpc_core::MutexLock lock(&mu_);
     while (!free_epoll1_handles_list_.empty()) {
       Epoll1EventHandle* handle = reinterpret_cast<Epoll1EventHandle*>(
           free_epoll1_handles_list_.front());
@@ -369,13 +385,14 @@ EventHandle* Epoll1Poller::CreateHandle(int fd, absl::string_view /*name*/,
                                         bool track_err) {
   Epoll1EventHandle* new_handle = nullptr;
   {
-    absl::MutexLock lock(&mu_);
+    grpc_core::MutexLock lock(&mu_);
     if (free_epoll1_handles_list_.empty()) {
       new_handle = new Epoll1EventHandle(fd, this);
     } else {
       new_handle = reinterpret_cast<Epoll1EventHandle*>(
           free_epoll1_handles_list_.front());
       free_epoll1_handles_list_.pop_front();
+      new_handle->ReInit(fd);
     }
   }
   ForkFdListAddHandle(new_handle);
@@ -389,7 +406,8 @@ EventHandle* Epoll1Poller::CreateHandle(int fd, absl::string_view /*name*/,
   ev.data.ptr = reinterpret_cast<void*>(reinterpret_cast<intptr_t>(new_handle) |
                                         (track_err ? 1 : 0));
   if (epoll_ctl(g_epoll_set_.epfd, EPOLL_CTL_ADD, fd, &ev) != 0) {
-    gpr_log(GPR_ERROR, "epoll_ctl failed: %s", strerror(errno));
+    gpr_log(GPR_ERROR, "epoll_ctl failed: %s",
+            grpc_core::StrError(errno).c_str());
   }
 
   return new_handle;
@@ -397,10 +415,13 @@ EventHandle* Epoll1Poller::CreateHandle(int fd, absl::string_view /*name*/,
 
 // Process the epoll events found by DoEpollWait() function.
 // - g_epoll_set.cursor points to the index of the first event to be processed
-// - This function then processes up-to MAX_EPOLL_EVENTS_PER_ITERATION and
-//   updates the g_epoll_set.cursor
-absl::Status Epoll1Poller::ProcessEpollEvents(
-    int max_epoll_events_to_handle, std::vector<EventHandle*>& pending_events) {
+// - This function then processes up-to max_epoll_events_to_handle and
+//   updates the g_epoll_set.cursor.
+// It returns true, it there was a Kick that forced invocation of this
+// function. It also returns the list of closures to run to take action
+// on file descriptors that became readable/writable.
+bool Epoll1Poller::ProcessEpollEvents(int max_epoll_events_to_handle,
+                                      Events& pending_events) {
   int64_t num_events = g_epoll_set_.num_events;
   int64_t cursor = g_epoll_set_.cursor;
   bool was_kicked = false;
@@ -414,43 +435,43 @@ absl::Status Epoll1Poller::ProcessEpollEvents(
       was_kicked = true;
     } else {
       Epoll1EventHandle* handle = reinterpret_cast<Epoll1EventHandle*>(
-          reinterpret_cast<intptr_t>(data_ptr) & ~static_cast<intptr_t>(1));
-      bool track_err =
-          reinterpret_cast<intptr_t>(data_ptr) & static_cast<intptr_t>(1);
+          reinterpret_cast<intptr_t>(data_ptr) & ~intptr_t{1});
+      bool track_err = reinterpret_cast<intptr_t>(data_ptr) & intptr_t{1};
       bool cancel = (ev->events & EPOLLHUP) != 0;
       bool error = (ev->events & EPOLLERR) != 0;
       bool read_ev = (ev->events & (EPOLLIN | EPOLLPRI)) != 0;
       bool write_ev = (ev->events & EPOLLOUT) != 0;
       bool err_fallback = error && !track_err;
-
-      handle->SetPendingActions(read_ev || cancel || err_fallback,
-                                write_ev || cancel || err_fallback,
-                                error && !err_fallback);
-      pending_events.push_back(handle);
+      if (handle->SetPendingActions(read_ev || cancel || err_fallback,
+                                    write_ev || cancel || err_fallback,
+                                    error && !err_fallback)) {
+        pending_events.push_back(handle);
+      }
     }
   }
   g_epoll_set_.cursor = cursor;
-  return was_kicked ? absl::Status(absl::StatusCode::kInternal, "Kicked")
-                    : absl::OkStatus();
+  return was_kicked;
 }
 
 //  Do epoll_wait and store the events in g_epoll_set.events field. This does
 //  not "process" any of the events yet; that is done in ProcessEpollEvents().
-//  See ProcessEpollEvents() function for more details.
-absl::Status Epoll1Poller::DoEpollWait(grpc_core::Timestamp deadline) {
+//  See ProcessEpollEvents() function for more details. It returns the number
+// of events generated by epoll_wait.
+int Epoll1Poller::DoEpollWait(EventEngine::Duration timeout) {
   int r;
-  int timeout = PollDeadlineToMillisTimeout(deadline);
   do {
     r = epoll_wait(g_epoll_set_.epfd, g_epoll_set_.events, MAX_EPOLL_EVENTS,
-                   timeout);
+                   static_cast<int>(
+                       grpc_event_engine::experimental::Milliseconds(timeout)));
   } while (r < 0 && errno == EINTR);
   if (r < 0) {
-    return absl::Status(absl::StatusCode::kInternal,
-                        absl::StrCat("epoll_wait: ", strerror(errno)));
+    grpc_core::Crash(absl::StrFormat(
+        "(event_engine) Epoll1Poller:%p encountered epoll_wait error: %s", this,
+        grpc_core::StrError(errno).c_str()));
   }
   g_epoll_set_.num_events = r;
   g_epoll_set_.cursor = 0;
-  return absl::OkStatus();
+  return r;
 }
 
 // Might be called multiple times
@@ -461,7 +482,7 @@ void Epoll1EventHandle::ShutdownHandle(absl::Status why) {
   // in parallel is not safe because some of the lockfree event types e.g, read,
   // write, error may-not have called SetShutdown when DestroyEvent gets
   // called in the OrphanHandle method.
-  absl::MutexLock lock(&mu_);
+  grpc_core::MutexLock lock(&mu_);
   HandleShutdownInternal(why, false);
 }
 
@@ -469,15 +490,15 @@ bool Epoll1EventHandle::IsHandleShutdown() {
   return read_closure_->IsShutdown();
 }
 
-void Epoll1EventHandle::NotifyOnRead(IomgrEngineClosure* on_read) {
+void Epoll1EventHandle::NotifyOnRead(PosixEngineClosure* on_read) {
   read_closure_->NotifyOn(on_read);
 }
 
-void Epoll1EventHandle::NotifyOnWrite(IomgrEngineClosure* on_write) {
+void Epoll1EventHandle::NotifyOnWrite(PosixEngineClosure* on_write) {
   write_closure_->NotifyOn(on_write);
 }
 
-void Epoll1EventHandle::NotifyOnError(IomgrEngineClosure* on_error) {
+void Epoll1EventHandle::NotifyOnError(PosixEngineClosure* on_error) {
   error_closure_->NotifyOn(on_error);
 }
 
@@ -487,29 +508,44 @@ void Epoll1EventHandle::SetWritable() { write_closure_->SetReady(); }
 
 void Epoll1EventHandle::SetHasError() { error_closure_->SetReady(); }
 
-absl::Status Epoll1Poller::Work(grpc_core::Timestamp deadline,
-                                std::vector<EventHandle*>& pending_events) {
+// Polls the registered Fds for events until timeout is reached or there is a
+// Kick(). If there is a Kick(), it collects and processes any previously
+// un-processed events. If there are no un-processed events, it returns
+// Poller::WorkResult::Kicked{}
+Poller::WorkResult Epoll1Poller::Work(
+    EventEngine::Duration timeout,
+    absl::FunctionRef<void()> schedule_poll_again) {
+  Events pending_events;
+  bool was_kicked_ext = false;
   if (g_epoll_set_.cursor == g_epoll_set_.num_events) {
-    auto status = DoEpollWait(deadline);
-    if (!status.ok()) {
-      return status;
+    if (DoEpollWait(timeout) == 0) {
+      return Poller::WorkResult::kDeadlineExceeded;
     }
   }
   {
-    absl::MutexLock lock(&mu_);
+    grpc_core::MutexLock lock(&mu_);
     // If was_kicked_ is true, collect all pending events in this iteration.
-    auto status = ProcessEpollEvents(
-        was_kicked_ ? INT_MAX : MAX_EPOLL_EVENTS_HANDLED_PER_ITERATION,
-        pending_events);
-    if (!status.ok()) {
+    if (ProcessEpollEvents(
+            was_kicked_ ? INT_MAX : MAX_EPOLL_EVENTS_HANDLED_PER_ITERATION,
+            pending_events)) {
       was_kicked_ = false;
+      was_kicked_ext = true;
+    }
+    if (pending_events.empty()) {
+      return Poller::WorkResult::kKicked;
     }
-    return status;
   }
+  // Run the provided callback.
+  schedule_poll_again();
+  // Process all pending events inline.
+  for (auto& it : pending_events) {
+    it->ExecutePendingActions();
+  }
+  return was_kicked_ext ? Poller::WorkResult::kKicked : Poller::WorkResult::kOk;
 }
 
 void Epoll1Poller::Kick() {
-  absl::MutexLock lock(&mu_);
+  grpc_core::MutexLock lock(&mu_);
   if (was_kicked_) {
     return;
   }
@@ -517,7 +553,7 @@ void Epoll1Poller::Kick() {
   GPR_ASSERT(wakeup_fd_->Wakeup().ok());
 }
 
-Epoll1Poller* GetEpoll1Poller(Scheduler* scheduler) {
+Epoll1Poller* MakeEpoll1Poller(Scheduler* scheduler) {
   static bool kEpoll1PollerSupported = InitEpoll1PollerLinux();
   if (kEpoll1PollerSupported) {
     return new Epoll1Poller(scheduler);
@@ -525,51 +561,54 @@ Epoll1Poller* GetEpoll1Poller(Scheduler* scheduler) {
   return nullptr;
 }
 
-}  // namespace posix_engine
+}  // namespace experimental
 }  // namespace grpc_event_engine
 
-#else /* defined(GRPC_LINUX_EPOLL) */
+#else  // defined(GRPC_LINUX_EPOLL)
 #if defined(GRPC_POSIX_SOCKET_EV_EPOLL1)
 
 namespace grpc_event_engine {
-namespace posix_engine {
+namespace experimental {
+
+using ::grpc_event_engine::experimental::EventEngine;
+using ::grpc_event_engine::experimental::Poller;
 
 Epoll1Poller::Epoll1Poller(Scheduler* /* engine */) {
-  GPR_ASSERT(false && "unimplemented");
+  grpc_core::Crash("unimplemented");
 }
 
-void Epoll1Poller::Shutdown() { GPR_ASSERT(false && "unimplemented"); }
+void Epoll1Poller::Shutdown() { grpc_core::Crash("unimplemented"); }
 
-Epoll1Poller::~Epoll1Poller() { GPR_ASSERT(false && "unimplemented"); }
+Epoll1Poller::~Epoll1Poller() { grpc_core::Crash("unimplemented"); }
 
 EventHandle* Epoll1Poller::CreateHandle(int /*fd*/, absl::string_view /*name*/,
                                         bool /*track_err*/) {
-  GPR_ASSERT(false && "unimplemented");
+  grpc_core::Crash("unimplemented");
 }
 
-absl::Status Epoll1Poller::ProcessEpollEvents(
-    int /*max_epoll_events_to_handle*/,
-    std::vector<EventHandle*>& /*pending_events*/) {
-  GPR_ASSERT(false && "unimplemented");
+bool Epoll1Poller::ProcessEpollEvents(int /*max_epoll_events_to_handle*/,
+                                      Events& /*pending_events*/) {
+  grpc_core::Crash("unimplemented");
 }
 
-absl::Status Epoll1Poller::DoEpollWait(grpc_core::Timestamp /*deadline*/) {
-  GPR_ASSERT(false && "unimplemented");
+int Epoll1Poller::DoEpollWait(EventEngine::Duration /*timeout*/) {
+  grpc_core::Crash("unimplemented");
 }
 
-absl::Status Epoll1Poller::Work(grpc_core::Timestamp /*deadline*/,
-                                std::vector<EventHandle*>& /*pending_events*/) {
-  GPR_ASSERT(false && "unimplemented");
+Poller::WorkResult Epoll1Poller::Work(
+    EventEngine::Duration /*timeout*/,
+    absl::FunctionRef<void()> /*schedule_poll_again*/) {
+  grpc_core::Crash("unimplemented");
 }
 
-void Epoll1Poller::Kick() { GPR_ASSERT(false && "unimplemented"); }
+void Epoll1Poller::Kick() { grpc_core::Crash("unimplemented"); }
 
 // If GRPC_LINUX_EPOLL is not defined, it means epoll is not available. Return
 // nullptr.
-Epoll1Poller* GetEpoll1Poller(Scheduler* /*scheduler*/) { return nullptr; }
+Epoll1Poller* MakeEpoll1Poller(Scheduler* /*scheduler*/) { return nullptr; }
 
-}  // namespace posix_engine
+}  // namespace experimental
 }  // namespace grpc_event_engine
 
-#endif /* defined(GRPC_POSIX_SOCKET_EV_EPOLL1) */
-#endif /* !defined(GRPC_LINUX_EPOLL) */
+#endif  // defined(GRPC_POSIX_SOCKET_EV_EPOLL1)
+#endif  // !defined(GRPC_LINUX_EPOLL)
