@@ -16,7 +16,7 @@
 
 #include <grpc/support/port_platform.h>
 
-#include <stdint.h>
+#include <stddef.h>
 
 #include <grpc/event_engine/event_engine.h>
 
@@ -24,9 +24,9 @@ namespace grpc_event_engine {
 namespace experimental {
 
 // Convert a duration to milliseconds
-int64_t Milliseconds(EventEngine::Duration d);
+size_t Milliseconds(EventEngine::Duration d);
 
 }  // namespace experimental
 }  // namespace grpc_event_engine
 
-#endif  // GRPC_CORE_LIB_EVENT_ENGINE_TIME_UTIL_H
\ No newline at end of file
+#endif  // GRPC_CORE_LIB_EVENT_ENGINE_TIME_UTIL_H
