@@ -1,20 +1,20 @@
-/*
- *
- * Copyright 2015 gRPC authors.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- *
- */
+//
+//
+// Copyright 2015 gRPC authors.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
+//
 
 #include <grpc/support/port_platform.h>
 
@@ -22,14 +22,15 @@
 
 #include <string.h>
 
+#include <new>
+
 #include <grpc/slice.h>
 #include <grpc/support/alloc.h>
 #include <grpc/support/log.h>
 
 #include "src/core/lib/gprpp/memory.h"
-#include "src/core/lib/gprpp/ref_counted.h"
 #include "src/core/lib/slice/slice_internal.h"
-#include "src/core/lib/slice/slice_refcount_base.h"
+#include "src/core/lib/slice/slice_refcount.h"
 
 char* grpc_slice_to_c_string(grpc_slice slice) {
   char* out = static_cast<char*>(gpr_malloc(GRPC_SLICE_LENGTH(slice) + 1));
@@ -51,8 +52,8 @@ grpc_slice grpc_slice_copy(grpc_slice s) {
 
 namespace grpc_core {
 
-/* grpc_slice_new support structures - we create a refcount object extended
-   with the user provided data pointer & destroy function */
+// grpc_slice_new support structures - we create a refcount object extended
+// with the user provided data pointer & destroy function
 class NewSliceRefcount : public grpc_slice_refcount {
  public:
   NewSliceRefcount(void (*destroy)(void*), void* user_data)
@@ -100,13 +101,13 @@ grpc_slice grpc_slice_new_with_user_data(void* p, size_t len,
 }
 
 grpc_slice grpc_slice_new(void* p, size_t len, void (*destroy)(void*)) {
-  /* Pass "p" to *destroy when the slice is no longer needed. */
+  // Pass "p" to *destroy when the slice is no longer needed.
   return grpc_slice_new_with_user_data(p, len, destroy, p);
 }
 
 namespace grpc_core {
-/* grpc_slice_new_with_len support structures - we create a refcount object
-   extended with the user provided data pointer & destroy function */
+// grpc_slice_new_with_len support structures - we create a refcount object
+// extended with the user provided data pointer & destroy function
 class NewWithLenSliceRefcount : public grpc_slice_refcount {
  public:
   NewWithLenSliceRefcount(void (*destroy)(void*, size_t), void* user_data,
@@ -127,7 +128,7 @@ class NewWithLenSliceRefcount : public grpc_slice_refcount {
   void (*user_destroy_)(void*, size_t);
 };
 
-/** grpc_slice_from_moved_(string|buffer) ref count .*/
+/// grpc_slice_from_moved_(string|buffer) ref count .
 class MovedStringSliceRefCount : public grpc_slice_refcount {
  public:
   explicit MovedStringSliceRefCount(UniquePtr<char>&& str)
@@ -147,6 +148,12 @@ class MovedCppStringSliceRefCount : public grpc_slice_refcount {
   explicit MovedCppStringSliceRefCount(std::string&& str)
       : grpc_slice_refcount(Destroy), str_(std::move(str)) {}
 
+  uint8_t* data() {
+    return reinterpret_cast<uint8_t*>(const_cast<char*>(str_.data()));
+  }
+
+  size_t size() const { return str_.size(); }
+
  private:
   static void Destroy(grpc_slice_refcount* arg) {
     delete static_cast<MovedCppStringSliceRefCount*>(arg);
@@ -205,10 +212,10 @@ grpc_slice grpc_slice_from_cpp_string(std::string str) {
     slice.data.inlined.length = str.size();
     memcpy(GRPC_SLICE_START_PTR(slice), str.data(), str.size());
   } else {
-    slice.data.refcounted.bytes =
-        reinterpret_cast<uint8_t*>(const_cast<char*>(str.data()));
-    slice.data.refcounted.length = str.size();
-    slice.refcount = new grpc_core::MovedCppStringSliceRefCount(std::move(str));
+    auto* refcount = new grpc_core::MovedCppStringSliceRefCount(std::move(str));
+    slice.data.refcounted.bytes = refcount->data();
+    slice.data.refcounted.length = refcount->size();
+    slice.refcount = refcount;
   }
   return slice;
 }
@@ -241,16 +248,16 @@ static grpc_slice sub_no_ref(const grpc_slice& source, size_t begin,
   GPR_ASSERT(end >= begin);
 
   if (source.refcount != nullptr) {
-    /* Enforce preconditions */
+    // Enforce preconditions
     GPR_ASSERT(source.data.refcounted.length >= end);
 
-    /* Build the result */
+    // Build the result
     subset.refcount = source.refcount;
-    /* Point into the source array */
+    // Point into the source array
     subset.data.refcounted.bytes = source.data.refcounted.bytes + begin;
     subset.data.refcounted.length = end - begin;
   } else {
-    /* Enforce preconditions */
+    // Enforce preconditions
     GPR_ASSERT(source.data.inlined.length >= end);
     subset.refcount = nullptr;
     subset.data.inlined.length = static_cast<uint8_t>(end - begin);
@@ -274,7 +281,7 @@ grpc_slice grpc_slice_sub(grpc_slice source, size_t begin, size_t end) {
            end - begin);
   } else {
     subset = grpc_slice_sub_no_ref(source, begin, end);
-    /* Bump the refcount */
+    // Bump the refcount
     if (subset.refcount != grpc_slice_refcount::NoopRefcount()) {
       subset.refcount->Ref();
     }
@@ -287,7 +294,7 @@ grpc_slice grpc_slice_split_tail_maybe_ref(grpc_slice* source, size_t split,
   grpc_slice tail;
 
   if (source->refcount == nullptr) {
-    /* inlined data, copy it out */
+    // inlined data, copy it out
     GPR_ASSERT(source->data.inlined.length >= split);
     tail.refcount = nullptr;
     tail.data.inlined.length =
@@ -296,7 +303,7 @@ grpc_slice grpc_slice_split_tail_maybe_ref(grpc_slice* source, size_t split,
            tail.data.inlined.length);
     source->data.inlined.length = static_cast<uint8_t>(split);
   } else if (source->refcount == grpc_slice_refcount::NoopRefcount()) {
-    /* refcount == NoopRefcount(), so we can just split in-place */
+    // refcount == NoopRefcount(), so we can just split in-place
     tail.refcount = grpc_slice_refcount::NoopRefcount();
     tail.data.refcounted.bytes = source->data.refcounted.bytes + split;
     tail.data.refcounted.length = source->data.refcounted.length - split;
@@ -306,13 +313,13 @@ grpc_slice grpc_slice_split_tail_maybe_ref(grpc_slice* source, size_t split,
     GPR_ASSERT(source->data.refcounted.length >= split);
     if (tail_length < sizeof(tail.data.inlined.bytes) &&
         ref_whom != GRPC_SLICE_REF_TAIL) {
-      /* Copy out the bytes - it'll be cheaper than refcounting */
+      // Copy out the bytes - it'll be cheaper than refcounting
       tail.refcount = nullptr;
       tail.data.inlined.length = static_cast<uint8_t>(tail_length);
       memcpy(tail.data.inlined.bytes, source->data.refcounted.bytes + split,
              tail_length);
     } else {
-      /* Build the result */
+      // Build the result
       switch (ref_whom) {
         case GRPC_SLICE_REF_TAIL:
           tail.refcount = source->refcount;
@@ -323,13 +330,13 @@ grpc_slice grpc_slice_split_tail_maybe_ref(grpc_slice* source, size_t split,
           break;
         case GRPC_SLICE_REF_BOTH:
           tail.refcount = source->refcount;
-          /* Bump the refcount */
+          // Bump the refcount
           if (tail.refcount != grpc_slice_refcount::NoopRefcount()) {
             tail.refcount->Ref();
           }
           break;
       }
-      /* Point into the source array */
+      // Point into the source array
       tail.data.refcounted.bytes = source->data.refcounted.bytes + split;
       tail.data.refcounted.length = tail_length;
     }
@@ -367,13 +374,13 @@ grpc_slice grpc_slice_split_head(grpc_slice* source, size_t split) {
   } else {
     GPR_ASSERT(source->data.refcounted.length >= split);
 
-    /* Build the result */
+    // Build the result
     head.refcount = source->refcount;
-    /* Bump the refcount */
+    // Bump the refcount
     if (head.refcount != grpc_slice_refcount::NoopRefcount()) {
       head.refcount->Ref();
     }
-    /* Point into the source array */
+    // Point into the source array
     head.data.refcounted.bytes = source->data.refcounted.bytes;
     head.data.refcounted.length = split;
     source->data.refcounted.bytes += split;
@@ -487,3 +494,9 @@ grpc_slice grpc_slice_dup(grpc_slice a) {
          GRPC_SLICE_LENGTH(a));
   return copy;
 }
+
+grpc_slice grpc_slice_ref(grpc_slice slice) {
+  return grpc_core::CSliceRef(slice);
+}
+
+void grpc_slice_unref(grpc_slice slice) { grpc_core::CSliceUnref(slice); }
