@@ -1,20 +1,20 @@
-/*
- *
- * Copyright 2015 gRPC authors.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- *
- */
+//
+//
+// Copyright 2015 gRPC authors.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
+//
 
 #ifndef GRPC_CORE_LIB_SURFACE_CHANNEL_H
 #define GRPC_CORE_LIB_SURFACE_CHANNEL_H
@@ -34,9 +34,10 @@
 #include "absl/strings/string_view.h"
 #include "absl/types/optional.h"
 
+#include <grpc/event_engine/event_engine.h>
 #include <grpc/event_engine/memory_allocator.h>
-#include <grpc/impl/codegen/compression_types.h>
-#include <grpc/impl/codegen/grpc_types.h>
+#include <grpc/grpc.h>
+#include <grpc/impl/compression_types.h>
 #include <grpc/slice.h>
 
 #include "src/core/lib/channel/channel_args.h"
@@ -56,24 +57,24 @@
 #include "src/core/lib/surface/channel_stack_type.h"
 #include "src/core/lib/transport/transport_fwd.h"
 
-/** The same as grpc_channel_destroy, but doesn't create an ExecCtx, and so
- * is safe to use from within core. */
+/// The same as grpc_channel_destroy, but doesn't create an ExecCtx, and so
+/// is safe to use from within core.
 void grpc_channel_destroy_internal(grpc_channel* channel);
 
-/** Create a call given a grpc_channel, in order to call \a method.
-    Progress is tied to activity on \a pollset_set. The returned call object is
-    meant to be used with \a grpc_call_start_batch_and_execute, which relies on
-    callbacks to signal completions. \a method and \a host need
-    only live through the invocation of this function. If \a parent_call is
-    non-NULL, it must be a server-side call. It will be used to propagate
-    properties from the server call to this new client call, depending on the
-    value of \a propagation_mask (see propagation_bits.h for possible values) */
+/// Create a call given a grpc_channel, in order to call \a method.
+/// Progress is tied to activity on \a pollset_set. The returned call object is
+/// meant to be used with \a grpc_call_start_batch_and_execute, which relies on
+/// callbacks to signal completions. \a method and \a host need
+/// only live through the invocation of this function. If \a parent_call is
+/// non-NULL, it must be a server-side call. It will be used to propagate
+/// properties from the server call to this new client call, depending on the
+/// value of \a propagation_mask (see propagation_bits.h for possible values)
 grpc_call* grpc_channel_create_pollset_set_call(
     grpc_channel* channel, grpc_call* parent_call, uint32_t propagation_mask,
     grpc_pollset_set* pollset_set, const grpc_slice& method,
     const grpc_slice* host, grpc_core::Timestamp deadline, void* reserved);
 
-/** Get a (borrowed) pointer to this channels underlying channel stack */
+/// Get a (borrowed) pointer to this channels underlying channel stack
 grpc_channel_stack* grpc_channel_get_channel_stack(grpc_channel* channel);
 
 grpc_core::channelz::ChannelNode* grpc_channel_get_channelz_node(
@@ -141,6 +142,7 @@ class Channel : public RefCounted<Channel>,
   absl::string_view target() const { return target_; }
   MemoryAllocator* allocator() { return &allocator_; }
   bool is_client() const { return is_client_; }
+  bool is_promising() const { return is_promising_; }
   RegisteredCall* RegisterCall(const char* method, const char* host);
 
   int TestOnlyRegisteredCalls() {
@@ -153,12 +155,18 @@ class Channel : public RefCounted<Channel>,
     return registration_table_.method_registration_attempts;
   }
 
+  grpc_event_engine::experimental::EventEngine* event_engine() const {
+    return channel_stack_->EventEngine();
+  }
+
  private:
-  Channel(bool is_client, std::string target, const ChannelArgs& channel_args,
+  Channel(bool is_client, bool is_promising, std::string target,
+          const ChannelArgs& channel_args,
           grpc_compression_options compression_options,
           RefCountedPtr<grpc_channel_stack> channel_stack);
 
   const bool is_client_;
+  const bool is_promising_;
   const grpc_compression_options compression_options_;
   std::atomic<size_t> call_size_estimate_;
   CallRegistrationTable registration_table_;
@@ -203,4 +211,4 @@ grpc_compression_options grpc_channel_compression_options(
 void grpc_channel_ping(grpc_channel* channel, grpc_completion_queue* cq,
                        void* tag, void* reserved);
 
-#endif /* GRPC_CORE_LIB_SURFACE_CHANNEL_H */
+#endif  // GRPC_CORE_LIB_SURFACE_CHANNEL_H
