@@ -1,20 +1,20 @@
-/*
- *
- * Copyright 2016 gRPC authors.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- *
- */
+//
+//
+// Copyright 2016 gRPC authors.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
+//
 
 #ifndef GRPC_CORE_LIB_SURFACE_CHANNEL_INIT_H
 #define GRPC_CORE_LIB_SURFACE_CHANNEL_INIT_H
@@ -22,8 +22,10 @@
 #include <grpc/support/port_platform.h>
 
 #include <functional>
+#include <utility>
 #include <vector>
 
+#include "src/core/lib/channel/channel_stack_builder.h"
 #include "src/core/lib/surface/channel_stack_type.h"
 
 #define GRPC_CHANNEL_INIT_BUILTIN_PRIORITY 10000
@@ -35,8 +37,6 @@
 
 namespace grpc_core {
 
-class ChannelStackBuilder;
-
 class ChannelInit {
  public:
   /// One stage of mutation: call functions against \a builder to influence the
@@ -72,10 +72,8 @@ class ChannelInit {
   };
 
   /// Construct a channel stack of some sort: see channel_stack.h for details
-  /// \a type is the type of channel stack to create
   /// \a builder is the channel stack builder to build into.
-  bool CreateStack(ChannelStackBuilder* builder,
-                   grpc_channel_stack_type type) const;
+  bool CreateStack(ChannelStackBuilder* builder) const;
 
  private:
   std::vector<Stage> slots_[GRPC_NUM_CHANNEL_STACK_TYPES];
@@ -83,4 +81,4 @@ class ChannelInit {
 
 }  // namespace grpc_core
 
-#endif /* GRPC_CORE_LIB_SURFACE_CHANNEL_INIT_H */
+#endif  // GRPC_CORE_LIB_SURFACE_CHANNEL_INIT_H
